
;------------------------------------------------------------------------------
;
;   programme to read the Helios ion/electron velocity distribution
;   functions, vdfp, vdfe and the corresponding velocity components.
;   program for the 3-D proton (2-D electron) velocity distribution function
;   and determination of the velocity ranges and maximum vdf value
;
;-----------------------------------------------------------------------------

;------------------- variables and arrays ------------------------------------

comment=strarr(1)

vdfp=dblarr(12,12,32)      ; protons/alpha-particles  3-D cubes
vxp =dblarr(12,12,32)
vyp =dblarr(12,12,32)
vzp =dblarr(12,12,32)
countp=intarr(12,12,32)

vdfe=dblarr(8,16)           ; electrons  2-D squares
vxe =dblarr(8,16)
vye =dblarr(8,16)
counte=intarr(8,16)

l = 12*12*32         ;= 4608  maximum proton arry size
print, 'lp=', l
l = 8*16             ;= 128   maximum electron array size
print, 'le=', l

ndimp = 4608
ndime = 128


;---------------------- definitions of variables ------------------------------
;
;	iearcarrot           = Carrington rotation number from Earth
;       ihelcarrot           = Carrington rotation number from spacecraft
;       i1a_3_on             = instrument i1a on (=0) or i3 on (=1)
;       index_shift
;       minus                = indicates a HDM file which contained bad
;                              data (frames), but could be handled as NDM file
;	mode                 = HDM or NMD
;       iperihelion_shift    = on or of (0 or 1)
;       idummy, iyesno       = dummy variable
;       indexi1.             = position of maximum value of instrument
;
;integer, ianzahl_e, ianzahl_p, idim, iearcarrot, ihelcarrot, i1a_3_on
;integer, index_shift, minus, mode
;integer, iperihelion_shift, idummy, ivesno
;integer, index_i1a(1), index_i1b(1)
;
;       earcarlat,earcarlng = Carrington longitude and latitude from Earth
;       earsdissun          = distance Earth-Sun (always zero, historical)
;       helcarlat,helcarlng = Carrington longitude and latitude from Helios
;       heldissun           = distance Helios S/C to Sun
;       vnorm, vrad    = S/C tangential (in ecliptic), radial velocity(AU/day)
;       angle               = spacecraft-Sun-Earth angle
;       np                  = proton density [cm^{-3}]
;       vxs,vys,vzs         = center of mass velocity (1-fluid velocity)
;       x,y,z,zfp,zfe       = dummy parameters used to read in the 3-D VDF
;       sinfi, cosfi        = sine and cosine of azimuth of proton velocity
;       sintheta, costheta  = sine and cosine of elevation of proton velocity
;       background_i1a,b    = background of instruments (here the first two
;                             and last two channels)
;
;real, earcarlat, earcarlng, earsdissun
;real, dissun, carlat, carlng, vnorm, vrad, angle
;real, np, vxs, vys, vzs, x, y, z, zfp, zfe
;real, sintheta, costheta, sinfi, cosfi, background_i1a, background_i1b

;       b                     = 3 components of magnetic field [nT]
;       bsig                  = stanadrad deviation of the 3 components
;                               of magnetic field [nT]
;       i1aint                = 32 onboard integrated enerry channels of
;                               protons insrtument i1a
;       i1bint                = 32 onboard integrated energy channels of
;                               protons insrtument i1b
;       i1av, i1bv            = corresponding velocities to i1aint, i1bint
;       i1aalp                = 3 dimenional array: density [cm^{-3}],
;                                velocity [km/s] and temperature [K] of alphas
;	    i1apro                = 5 dimenional array: density [cm^{-3}],
;                                velocity [km/s], temperature [K] of protons
;    	i1bpro                = 3 dimenional array: density [cm^{-3}],
;                                velocity [km/s], temperature [K] of protons

b     =dblarr(3)
bsig  =dblarr(3)
i1aint=dblarr(32)
i1av  =dblarr(32)
i1bint=dblarr(32)
i1bv  =dblarr(32)
i1apro=fltarr(5)
i1aalp=fltarr(3)
i1bpro=fltarr(3)
;
;
;       now read in the data. The first data set, the correction from
;       Pizzo et al. and for the finite solar diameter will not be used
;       (the first three records).
;
;	All other parameters are read in. For a definition see top.
;
;   A 'read' without a parameter reads of a 'comment' in the data file
;
print, 'enter reading'
;
;---------------------------------------------------------------------------

readf,1, comment
print, comment
readf,1, comment
print, comment
readf,1, comment
print, comment
readf,1, imode, ishift, iperihelion_shift, minus, i1a_3_on
print, imode, ishift, iperihelion_shift, minus, i1a_3_on
readf,1, heldissun, helcarlng, helcarlat, helcarrot
print, heldissun, helcarlng, helcarlat, helcarrot
readf,1, eardissun, earcarlng, earcarlat, angle, earcarrot
print, eardissun, earcarlng, earcarlat, angle, earcarrot
readf,1, vrad, vnorm
print, vrad, vnorm
readf,1, i1apro
print, i1apro
readf,1, i1aalp
print, i1aalp
readf,1, i1bpro
print, i1bpro
readf,1, b, bsig
print, b, bsig
readf,1, comment
print, comment
readf,1, i1aint
print, i1aint
readf,1, comment
print, comment
readf,1, i1av
print, i1av
readf,1, comment
print, comment
readf,1, i1bint
print, i1bint
readf,1, comment
print, comment
readf,1, i1bv
print, i1bv
readf,1, comment
print, comment
readf,1, comment
print, comment
readf,1, comment
print, comment


;print, 'Integrated data'
;print, i1aint
;print, i1av
;print, i1bint
;print, i1bv

;---------------------------------------------------------------------------
;       Define velocity with respect to center of mass (1-D velocity).
;       Transform from degree to radian. Calculate center of mass
;       velocity 'v.s'
;
pi180 = 3.141592653589793238462643d0/180.d0
;print, pi180

	sinfi = sin(i1apro(3) * pi180)
	cosfi = cos(i1apro(3) * pi180)
	sintheta = sin(i1apro(4) * pi180)
	costheta = cos(i1apro(4) * pi180)
	vxs = i1apro(1) * cosfi * costheta
	vys = i1apro(1) * sinfi * costheta
	vzs = i1apro(1) * sintheta

print, 'solar wind (proton) velocity components'
print, vxs,vys,vzs


;---------------------------- Read VDFs --------------------------------------
;
;       now loop through the distribution fucntion. First read in the
;       the indices i,j,k of distribution function and the velocity
;       components. The indices are: i for azimuth, j for elevation
;       and k for voltage. The dummy variables z,x,y,zfp,zfe contain the
;       value of the distribution function, and the respective velocity
;       components. The variable 'dummy' contains the count rate, which can
;       be used for statistics. The dummy variables are then transformed
;       in the center of mass system and written to the velocity arrays
;       wx,wy,wz. If an error occurs during read, the program
;       will continue at label electrons. The next data will be the
;       electrons, which are handled later.
;

;========================= protons ===========================================

for mi=0,11 do begin
for mj=0,11 do begin
for mk=0,31 do begin
vdfp(mi,mj,mk) = 0.d0
vxp(mi,mj,mk) = 0.d0
vyp(mi,mj,mk) = 0.d0
vzp(mi,mj,mk) = 0.d0
countp(mi,mj,mk) = 0
endfor
endfor
endfor

for n_p=0, ndimp do begin
        on_ioerror, electrons      ; jump to electron data
        readf,1, i, j, k, zfp, dummy, x, y, z
        i=fix(i)
        j=fix(j)
        k=fix(k)
        dummy = fix(dummy)
        ii= i-1
        jj= j-1
        kk= k-1
        ;print, ii, jj, kk, dummy,'    ', zfp, x, y, z
        vdfp(ii,jj,kk) = zfp
        vxp(ii,jj,kk) = x
        vyp(ii,jj,kk) = y
        vzp(ii,jj,kk) = z
        countp(ii,jj,kk)= dummy
endfor

;============================ electrons ===========================
;
;       Now read electron data. Procedure similar to that of protons.
;       First print one coment line, then loop: Structure: i,j indices
;       of the distribution function, zfe value of distribution function,
;       dummy is the count rate, x,y are the electron velocities


electrons: readf,1, comment
           print, comment
;           readf,1, comment
;           print, comment

for mi=0,7 do begin
for mj=0,15 do begin
vdfe(mi,mj) = 0.d0
vxe(mi,mj) = 0.d0
vye(mi,mj) = 0.d0
counte(mi,mj) = 0
endfor
endfor

for n_e=0, ndime do begin
    while not EOF(1) do begin
        readf,1, i, j, zfe, dummy, x, y
        i = fix(i)
        j = fix(j)
        dummy = fix(dummy)
        ii = i - 1
        jj = j - 1
        ;print, ii, jj, dummy,'    ', zfe, x, y
        vdfe(ii,jj) = zfe
        vxe(ii,jj) = x
        vye(ii,jj) = y
        counte(ii,jj) = dummy
    endwhile
endfor

;close,1

; ---------------------- end reading the data from file --------------------

;------------- determine the velocity ranges and maxima -----------------------

print, 'velocity ranges and maxima'

; protons

vdfpmax = max(vdfp)
vxpmin = min(vxp)
vxpmax = max(vxp)
vypmin = min(vyp)
vypmax = max(vyp)
vzpmin = min(vzp)
vzpmax = max(vzp)
print, vdfpmax
print, vxpmin,vxpmax,vypmin,vypmax,vzpmin,vzpmax

countpmax = max(countp)
print, countpmax

; electrons

vdfemax = max(vdfe)
vxemin = min(vxe)
vxemax = max(vxe)
vyemin = min(vye)
vyemax = max(vye)
print, vdfemax
;print, imaxe,jamxe
print, vxemin,vxemax,vyemin,vyemax

countemax = max(counte)
print, countemax

;------------ Limit the domain of the ion and electron ---------------------
;------------ velocity distribution functions ------------------------------
;------------- The arrays are defined above --------------------------------
;------------- determined in the read subroutine ---------------------------
;------ determine the velocity domain with VDF above 1-2-count level --------
;------ determine proton indices of the new boundaries ----------------------

ptoti=intarr(12)
ptotj=intarr(12)
ptotk=intarr(32)

for k=0,31 do begin
              ptotk(k) = total(countp(*,*,k))
              ;print, k, ptotk(k)
              endfor
for j=0,11 do begin
              ptotj(j) = total(countp(*,j,*))
              ;print, j, ptotj(j)
              endfor
for i=0,11 do begin
              ptoti(i) = total(countp(i,*,*))
              ;print, i, ptoti(i)
              endfor

; electrons

etoti=intarr(8)
etotj=intarr(16)

for i=0,7  do begin
              etoti(i) = total(counte(i,*))
              ;print, i, etoti(i)
              endfor

for j=0,15 do begin
              etotj(j) = total(counte(*,j))
              ;print, j, etotj(j)
              endfor

print, 'determine proton array limits'

;--- Use full electron arrays but shrink proton arrays -----------------------


              for k=0,31 do begin
              if ptotk(k) GT 5 then GOTO, continue_k1
              endfor
continue_k1:  ;print, k, ptotk(k)
              klow = k

              for k=0,31 do begin
              kk = 31-k
              if ptotk(kk) GT 5 then GOTO, continue_k2
              endfor
continue_k2:  ;print, kk, ptotk(kk)
              khig = kk

              for j=0,11 do begin
              if ptotj(j) GT 2 then GOTO, continue_j1
              endfor
continue_j1:  ;print, j, ptotj(j)
              jlow = j

              for j=0,11 do begin
              jj = 11-j
              if ptotj(jj) GT 2 then GOTO, continue_j2
              endfor
continue_j2:  ;print, jj, ptotj(jj)
              jhig = jj

              for i=0,11 do begin
              if ptoti(i) GT 2 then GOTO, continue_i1
              endfor

continue_i1:  ;print, i, ptoti(i)
              ilow = i

              for i=0,11 do begin
              ii = 11-i
              if ptoti(ii) GT 2 then GOTO, continue_i2
              endfor

continue_i2:  ;print, ii, ptoti(ii)
              ihig = ii

print, ilow, ihig, jlow, jhig, klow, khig

vxlow = 0.
vxhig = 0.
vylow = 0.
vyhig = 0.
vzlow = 0.
vzhig = 0.

for i=ilow, ihig do begin
for j=jlow, jhig do begin
for k=klow, khig do begin
if vxp(i,j,k) LT vxlow then vxlow = vxp(i,j,k)
if vxp(i,j,k) GT vxhig then vxhig = vxp(i,j,k)
if vyp(i,j,k) LT vxlow then vylow = vyp(i,j,k)
if vyp(i,j,k) GT vxlow then vyhig = vyp(i,j,k)
if vzp(i,j,k) LT vxlow then vzlow = vzp(i,j,k)
if vzp(i,j,k) GT vxlow then vzhig = vzp(i,j,k)
endfor
endfor
endfor

print, vxlow, vxhig, vylow, vyhig, vzlow, vzhig
print, vxpmin, vxpmax, vypmin, vypmax, vzpmin, vzpmax

;  Restrict array sizes

iend = ihig - ilow
jend = jhig - jlow
kend = khig - klow
idimen = iend + 1
jdimen = jend + 1
kdimen = kend + 1
number= idimen*jdimen*kdimen
print, iend, jend, kend
print, idimen, jdimen, kdimen, number

fp=dblarr(idimen,jdimen,kdimen) ; protons/alpha-particles reduced 3-D cubes
xp=dblarr(idimen,jdimen,kdimen)
yp=dblarr(idimen,jdimen,kdimen)
zp=dblarr(idimen,jdimen,kdimen)
cp=intarr(idimen,jdimen,kdimen)

for i=0, iend do begin
ii = ilow + i
for j=0, jend do begin
jj = jlow + j
for k=0, kend do begin
kk = klow + k
fp(i,j,k) = 0.d0
cp(i,j,k) = 0
xp(i,j,k) = 0.d0
yp(i,j,k) = 0.d0
zp(i,j,k) = 0.d0
if countp(ii,jj,kk) LT 1 then GOTO, continue_10
xp(i,j,k) = vxp(ii,jj,kk)
yp(i,j,k) = vyp(ii,jj,kk)
zp(i,j,k) = vzp(ii,jj,kk)
fp(i,j,k) = vdfp(ii,jj,kk)
cp(i,j,k) = countp(ii,jj,kk)
print, xp(i,j,k),yp(i,j,k),zp(i,j,k),fp(i,j,k),'  ',cp(i,j,k)
continue_10:
endfor
endfor
endfor

;--- comment: the velocity components are set to zero for vanishing VDF ------

faint=dblarr(kdimen)
va   =dblarr(kdimen)
fbint=dblarr(kdimen)
vb   =dblarr(kdimen)

;--------------------- Reduce the size of the integrated spectra --------------

for k=0, kend do begin
kk = klow + k
;print, k, kk
faint(k)= i1aint(kk)
va(k)   =   i1av(kk)
fbint(k)= i1bint(kk)
vb(k)   =   i1bv(kk)
endfor

; ---------- Determine the summed-up (over i and j) VDF of protons -----------

fsum=dblarr(kdimen)
for k=0,kend do begin
                fsum(k) = total(fp(*,*,k))
                print, k, fsum(k)
                endfor
                fnorm = idimen*jdimen
                print, fnorm
                ;fsum=fsum/fnorm
print, faint
print, va
print, fbint
print, vb
;----- Go into the 'wind' frame of reference ------------------------------

wx=dblarr(idimen,jdimen,kdimen)
wy=dblarr(idimen,jdimen,kdimen)
wz=dblarr(idimen,jdimen,kdimen)

for i=0, iend do begin
for j=0, jend do begin
for k=0, kend do begin
if cp(i,j,k) LT 1 then GOTO, continue_20
wx(i,j,k) = xp(i,j,k) - vxs
wy(i,j,k) = yp(i,j,k) - vys
wz(i,j,k) = zp(i,j,k) - vzs
print, wx(i,j,k),wy(i,j,k),wz(i,j,k),fp(i,j,k),'  ',cp(i,j,k)
continue_20:
endfor
endfor
endfor

;------------- plot some spectra -------------------------------------------
plot, va, faint
oplot, vb, fbint

;vdfsum = dblarr(kdimen)
;for k=0, kend do begin
;vdfsum(k) = total(vdfp(*,*,k))
;endfor
;plot, vdfsum

;vdfcut = dblarr(idimen,jdimen)
;for k=0, kend do begin
;print, k
;  for i=0, iend do begin
; for j=0, jend do begin
;  vdfcut(i,j) = vdfp(i,j,k)
;  endfor
; endfor
;  contour, vdfcut
;endfor

;-------------------------- end programme -----------------------------------

end

